<?php

namespace Drupal\resourcespace_integration\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Drupal\image\Entity\ImageStyle;
use Drupal\media\Entity\Media;
use Symfony\Component\HttpFoundation\Request;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityStorageException;


class ResourceSpaceController extends ControllerBase
{

  public function __construct() {
  }
    /**
     * Connect to ResourceSpace API, authenticate, and list resources.
     */
    public function listResources(Request $request)
    {
        // Load configuration.
        $config = \Drupal::config('resourcespace_integration.settings');
        $apiUrl = $config->get('resourcespace_url');
        $username = $config->get('resourcespace_username');
        $apiKey = $config->get('resourcespace_api_key');

        if (!$apiUrl || !$username || !$apiKey) {
            return [
                '#type' => 'markup',
                '#markup' => '<p>ResourceSpace credentials are not configured. Please update the settings.</p>',
            ];
        }

        $searchTerm = $request->query->get('search', '');

        $query = "user={$username}&function=search_get_previews&getsizes=scr&previewext=jpg&search=" . urlencode($searchTerm);

        // Generate the signature.
        $signature = hash('sha256', $apiKey . $query);

        $fullUrl = "{$apiUrl}?{$query}&sign={$signature}";


        // Make the API request.
        $response = $this->makeApiRequest($fullUrl);

        // Decode the JSON response.
        $resources = json_decode($response, true);

        if ($resources && is_array($resources)) {
            foreach ($resources as $resource) {
                $this->createMediaEntity($resource);
            }
        }
        // Pass the data to the template.
        return [
            '#theme' => 'resource_space_listing',
            '#resources' => $resources,
            '#search_term' => $searchTerm,
        ];
    }

     /**
     * Connect to ResourceSpace API, authenticate, and list resources for cron.
     */
    public function getResources($search="")
    {
        // Load configuration.
        $config = \Drupal::config('resourcespace_integration.settings');
        $apiUrl = $config->get('resourcespace_url');
        $username = $config->get('resourcespace_username');
        $apiKey = $config->get('resourcespace_api_key');

        if (!$apiUrl || !$username || !$apiKey) {
            return [
                '#type' => 'markup',
                '#markup' => '<p>ResourceSpace credentials are not configured. Please update the settings.</p>',
            ];
        }

        $searchTerm = $search;

        $query = "user={$username}&function=search_get_previews&getsizes=scr&previewext=jpg&search=" . urlencode($searchTerm);

        // Generate the signature.
        $signature = hash('sha256', $apiKey . $query);
        // Full API URL with the signature.
        $fullUrl = "{$apiUrl}?{$query}&sign={$signature}";
        // Make the API request.
        $response = $this->makeApiRequest($fullUrl);

        // Decode the JSON response.
        $resources = json_decode($response, true);
        $ref_ids = [];
        if ($resources && is_array($resources)) {
            foreach ($resources as $resource) {
                $this->createMediaEntity($resource);
                $ref_ids[] = $resource['ref'];
            }
        }
        $this->deleteMedias($ref_ids);
    }

    private function deleteMedias($external_ref_ids){
        $database = \Drupal::database();
        $query = $database->select('media__field_ref_id', 'mfr')
            ->fields('mfr', ['entity_id', 'field_ref_id_value'])
            ->condition('field_ref_id_value', NULL, 'IS NOT NULL');
        $results = $query->execute()->fetchAllKeyed(1, 0);

        $missing_ref_ids = array_diff(array_keys($results),$external_ref_ids);
        $media_ids_to_delete = [];
        foreach ($missing_ref_ids as $ref_id) {
            if (isset($results[$ref_id])) {
                $media_ids_to_delete[] = $results[$ref_id];
            }
        }
        if (!empty($media_ids_to_delete)) {
            $media_storage = \Drupal::entityTypeManager()->getStorage('media');
            $media_storage->delete($media_storage->loadMultiple($media_ids_to_delete));
        }
    }


    public function getResoucePath($ref_id,$extension){

        // Load configuration.
        $config = \Drupal::config('resourcespace_integration.settings');
        $apiUrl = $config->get('resourcespace_url');
        $username = $config->get('resourcespace_username');
        $apiKey = $config->get('resourcespace_api_key');

        if (!$apiUrl || !$username || !$apiKey) {
            return [];
        }


        $query = "user={$username}&function=get_resource_path&external=true&extension={$extension}&ref=" . urlencode($ref_id);

        // Generate the signature.
        $signature = hash('sha256', $apiKey . $query);

        // Full API URL with the signature.
        $fullUrl = "{$apiUrl}?{$query}&sign={$signature}";
        // Make the API request.
        $response = $this->makeApiRequest($fullUrl);

        // Decode the JSON response.
        $resources = json_decode($response, true);
        return $resources;
    }

    /**
     * Authenticate with ResourceSpace and obtain a session API key.
     */
    private function authenticate($private_key, $username, $query)
    {

        $signature = hash('sha256', $private_key . $query);
        return $signature;
    }


    /**
     * Creates media entities for each image.
     */
    public function createMediaEntity($image_data)
    {

        // Log the incoming image data for debugging.
        $image_url = $this->getResoucePath($image_data['ref'],$image_data['file_extension']);
        $ref_id = $image_data['ref'];
        $extension = $image_data['file_extension'];
        $modified_at = $image_data['file_modified'];
        if ($image_url && $ref_id && $extension) {

            try {
                // Check if the media entity with the same ref_id already exists.
                $query = \Drupal::entityQuery('media')
                    ->condition('bundle', 'image')
                    ->condition('field_ref_id', $ref_id)
                    ->accessCheck(false);
                $existing_media_ids = $query->execute();

                if (!empty($existing_media_ids)) {
                    $name = $image_data['field8'] . "." . $extension;
                    $this->checkForUpdate($image_url,$name,$ref_id,$extension,$modified_at);
                    return;
                }
                $file = $this->save_image_to_media_library($image_url, $image_data['field8'] . "." . $extension);
                // Step 2: Generate the thumbnail using an image style.
                $image_style = ImageStyle::load('thumbnail');

                $image_uri = $file->getFileUri();
                $destination_uri = $image_style->buildUri($file->uri->value);
                $image_style->createDerivative($image_uri, $destination_uri);

                // Create the media entity.
                \Drupal::logger('resourcespace_integration')->info($image_url);
                $media = \Drupal\media\Entity\Media::create([
                    'bundle' => 'image',
                    'name' => $file->getFilename(),
                    'field_ref_id' => $ref_id,
                    'field_resource_date' => $modified_at,
                    'field_remote_image_url' => $image_url,
                    'field_media_image' => [
                        'target_id' => $file->id(),
                        'alt' => 'ResourceSpace image',
                    ],
                ]);
                $media->save();

            } catch (\Exception $e) {
                \Drupal::logger('resourcespace_integration')->error('Error creating media entity: @message', ['@message' => $e->getMessage()]);
            }
        } else {
            \Drupal::logger('resourcespace_integration')->error('Invalid image URL or missing ref_id.');
        }
    }

    /**
     * Check for update of media entities.
     */
    private function checkForUpdate($image_url,$name,$ref_id,$extension,$modified_at){
        $query = \Drupal::entityQuery('media')
        ->condition('bundle', 'image')  // Filter by image bundle.
        ->condition('field_ref_id', $ref_id)  // Filter by field_ref_id.
        ->accessCheck(TRUE);  // Skip access control check.

        $media_ids = $query->execute();  // Execute the query to get the media entity IDs.

        if (!empty($media_ids)) {
            // Load the media entities by their IDs.
            $media_storage = \Drupal::entityTypeManager()->getStorage('media');
            $media_entities = $media_storage->loadMultiple($media_ids);

            if (empty($modified_at)) {
                // Use the current date and time if given date is empty.
                $modified_at = new DrupalDateTime('now');
              } else {
                // Convert the given date to a DrupalDateTime object if it is provided.
                $modified_at = new DrupalDateTime($modified_at);
              }
            // Loop through the media entities.
            foreach ($media_entities as $media) {
            // Get the 'field_modified_at' value.
            $field_modified_at = $media->get('field_modified_at')->value;
            $modified_at_datetime = new DrupalDateTime($field_modified_at);
            if ($modified_at_datetime < $modified_at) {
                $media->set('name', $name);  // Set new name.

                $media->set('field_modified_at', $modified_at->format('Y-m-d H:i:s'));  // Set new modified date.

                $file = $this->save_image_to_media_library($url, $name);
                $image_style = ImageStyle::load('thumbnail');

                $image_uri = $file->getFileUri();
                $destination_uri = $image_style->buildUri($file->uri->value);
                $image_style->createDerivative($image_uri, $destination_uri);
                if ($file) {
                    $media->set('field_media_image', $file->id());
                    $media->save();
                }
             }
            }
        } else {
        }
    }

    /**
     * Creates media entities for each image.
     */
    public function createMediaEntityForAjax($url,$name,$ref_id,$extension,$modified_at)
    {

        // Log the incoming image data for debugging.
        $image_url = $this->getResoucePath($ref_id,$extension);
        if ($image_url && $ref_id && $extension) {

            try {
                // Check if the media entity with the same ref_id already exists.
                $query = \Drupal::entityQuery('media')
                    ->condition('bundle', 'image')
                    ->condition('field_ref_id', $ref_id)
                    ->accessCheck(false);
                $existing_media_ids = $query->execute();

                if (!empty($existing_media_ids)) {
                    return;
                }
                $file = $this->save_image_to_media_library($image_url, $name . "." . $extension);
                // Step 2: Generate the thumbnail using an image style.
                $image_style = ImageStyle::load('thumbnail');

                $image_uri = $file->getFileUri();
                $destination_uri = $image_style->buildUri($file->uri->value);
                $image_style->createDerivative($image_uri, $destination_uri);

                // Create the media entity.
                $media = \Drupal\media\Entity\Media::create([
                    'bundle' => 'image',
                    'name' => $file->getFilename(),
                    'field_ref_id' => $ref_id,
                    'field_resource_date' => $modified_at,
                    'field_remote_image_url' => $image_url,
                    'field_media_image' => [
                        'target_id' => $file->id(),
                        'alt' => 'ResourceSpace image',
                    ],
                ]);
                $media->save();
                return $media;
            } catch (\Exception $e) {
                 \Drupal::logger('resourcespace_integration')->error('Error creating media entity: @message', ['@message' => $e->getMessage()]);
            }
        } else {
        }
    }

    public function save_image_to_media_library($image_url, $filename)
    {
        // Define the target directory in the public file system.
        $destination = 'public://images/';
        \Drupal::service('file_system')->prepareDirectory($destination, FileSystemInterface::CREATE_DIRECTORY);

        // Get the image data.
        $image_data = file_get_contents($image_url);
        if ($image_data === false) {
            return null;
        }

        // Generate a unique file name.
        $file_uri = $destination . $filename;

        // Save the file.
        $file_system = \Drupal::service('file_system');
        $file_path = $file_system->saveData($image_data, $file_uri, FileSystemInterface::EXISTS_REPLACE);

        if (!$file_path) {
            return null;
        }

        // Create a File entity.
        $file = File::create([
            'uri' => $file_path,
            'status' => 1,
        ]);
        $file->save();

        return $file;
    }


    private function makeApiRequest($url)
    {
        // Get the HTTP client service.
        $httpClient = \Drupal::httpClient();

        try {
            // Make the GET request.
            $response = $httpClient->get($url, [
                'verify' => false,
                'timeout' => 40,
            ]);

            // Return body as a string.
            return $response->getBody()->getContents();
        } catch (RequestException $e) {
            return null;
        }
    }

    /**
     * Handles AJAX requests.
     */
    public function ajaxCallback() {
       // Decode the JSON payload.
    $content = json_decode(\Drupal::request()->getContent(), TRUE);

    // Retrieve the selected items from the decoded JSON.
    $selected_items = $content['selected_items'] ?? [];

    if (empty($selected_items)) {
      return new JsonResponse(['status' => 'error', 'message' => 'No items selected.'], 400);
    }

    $processed_media = [];
    foreach ($selected_items as $item) {
      $ref_id = $item['id'] ?? NULL; // This is your `field_ref_id`.
      $name = $item['name'] ?? NULL;
      $url = $item['url'] ?? NULL;
      $modified_at = $item['modified'] ?? NULL;
      $extension = $item['extension'] ?? NULL;

      if (!$ref_id || !$name || !$url) {
        continue; // Skip items with incomplete data.
      }

      // Check if a media entity with this field_ref_id already exists.
      $media_storage = \Drupal::entityTypeManager()->getStorage('media');
      $existing_media = $media_storage->loadByProperties(['field_ref_id' => $ref_id]);

      if ($existing_media) {
        // Media exists, update the first match.
        $media = reset($existing_media);
        $media->setName($name);
        $file = $this->save_image_to_media_library($url, $name . "." . $extension);
        if ($file) {
            $media->set('field_modified_at', $modified_at);
            $media->set('field_media_image', $file->id());

            $media->save();

            $processed_media[] = ['id' => $media->id(), 'action' => 'updated'];
          }
      } else {

        $media = $this->createMediaEntityForAjax($url,$name,$ref_id,$extension,$modified_at);
        $processed_media[] = ['id' => $media->id(), 'action' => 'created'];
      }
    }

    // Return a success response.
    return new JsonResponse([
      'status' => 'success',
      'message' => 'Processed ' . count($processed_media) . ' media entities.',
      'processed_media' => $processed_media,
    ]);
    }

}
